# Conversion of modified frcmod files into SMIRNOFF OFFXML format

Amber parm99 with a multitude of additional parameters in parm@Frosst can be effectively used on a wide spectrum of pharmaceutically relevant small molecules. An attempt is made here to convert these into smirnoff format.
These are all files associated with converting and testing the combined SMIRNOFF  *smirnoff99Frosst*


## Manifest
* `convert_frcmod.py` - python script to convert a frcmod file (with SMIRKS i.e. `example.frcmod`) into a SMIRNOFF ffxml file by parsing the parameters and inserting them into a template offxml.
* `example.frcmod` - example modified frcmod file which contains SMIRKS in the place of normal AMBER atom types in the parameter definitions
* `template.offxml` - template SMIRNOFF OFFXML file which the parameters from the frcmod file will be inserted into; already contains relevant force sections but not the paramters themselves.
* `example.offxml` - converted SMIRNOFF OFFXML as generated by convert_frcmod.py
* `example_plusgeneric.offxml` - example which takes a generated file created by convert_frcmod.py and adds some generic types as parameter 0 under each force that ensure the result covers all parameter types.
* `parameter_usage.ipynb` - example notebook (requested and updated by Bayly) which shows how to determine which molecules do/do not contain a specific parameter and then visualized those molecules and where that parameter is assigned. Used to check the use of generic bond, angle, or torsion SMIRKS in the offxml to determine what chemistry is not covered by the relevant SMIRNOFF
* smirnoffishFrcmod.parm99Frosst.txt - the key source file for the smirnoff offxml file, this file has the format of an amber .frcmod file except the initial field giving bonded amber atomtypes is replaced with a smirks string for use with the smirnoff format for that parameter.
* smirnoff99Frosst.offxml - this is the SMIRNOFF format file containing the best efforts of C.I.Bayly during his sabbatical in summer 2016 to convert amber parm99.dat plus the parm@Frosst frcmod file into a single small-molecule SMIRNOFF offxml.
* smirnoffishFrcmod.parm99Frosst_AlkEthOH.txt - a version of smirnoffishFrcmod.parm99Frosst.txt which has been extended to include the same parameters as parm@frosst on the AlkEthOH set, including in cases where Bayly deliberately deviated from it (e.g. for the CT-OH-HO angle/force constant where he went to the generic). Created by David Mobley for energy checking purposes.

## Usage Examples

#### convert_frcmod.py

This python script convers the frcmod type files, with SMIRKS strings instead of combined atomtypes into a SMIRNOFF offxml file. It can be imported or called from the commandline.

```
Usage:     Convert specified SMIRKS-ified AMBER frcmod file into SMIRNOFF OFFXML format, inserting converted parameters into a template OFFXML file and writing to a new output file.

    usage: convert_frcmod.py --frcmod test.frcmod --template template.offxml --xml test.offxml


Options:
  -h, --help            show this help message and exit
  -f INFILE, --frcmod=INFILE
                        Name of input smirks-ified frcmod file.
  -t INXML, --template=INXML
                        Name of template SMIRNOFF ffxml file.
  -o OUTXML, --xml=OUTXML
                        Name of output SMIRNOFF ffxml file.
```

For exmple, smirnoff99Fross.offxml was created by calling:

```
python convert_frcmod.py -f smirnoffishFrcmod.parm99Frosst.txt -t template.offxml -o smirnoff99Frosst.offxml
```

#### parameter_usage.ipynb

parameter_usage.ipynb - this ipython notebook originally written by David Mobley carries out several independent steps:
a) Reading in a smirnoffishFrcmod file (here smirnoffishFrcmod.parm99Frosst.txt) and converting it to SMIRNOFF99Frosst
b) Reading in a database of molecules and applying to them the smirnoff offxml from part (a).
c) Analyzing the result of (b) to see which molecules get which parameters, including a depiction function which will depict a molecule highlighting the atoms associated with a specific parameter.
